import { NotFoundException } from '@nestjs/common';
import { QueryHandler } from '@nestjs/cqrs';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { Messager } from '../../adapters/secondaries/messager';
import { FindConfigurationByUuidQuery } from '../../queries/find-configuration-by-uuid.query';
import { Configuration } from '../entities/configuration';

@QueryHandler(FindConfigurationByUuidQuery)
export class FindConfigurationByUuidUseCase {
  constructor(
    private readonly repository: ConfigurationRepository,
    private readonly messager: Messager,
  ) {}

  execute = async (
    findConfigurationByUuid: FindConfigurationByUuidQuery,
  ): Promise<Configuration> => {
    try {
      const configuration = await this.repository.findOneByUuid(
        findConfigurationByUuid.uuid,
      );
      if (!configuration) throw new NotFoundException();
      return configuration;
    } catch (error) {
      this.messager.publish(
        'logging.configuration.read.warning',
        JSON.stringify({
          query: findConfigurationByUuid,
          error,
        }),
      );
      throw error;
    }
  };
}
