import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { Messager } from '../../adapters/secondaries/messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { UpdateConfigurationCommand } from '../../commands/update-configuration.command';
import { UpdateConfigurationRequest } from '../dtos/update-configuration.request';
import { Configuration } from '../entities/configuration';

@CommandHandler(UpdateConfigurationCommand)
export class UpdateConfigurationUseCase {
  constructor(
    private readonly repository: ConfigurationRepository,
    private readonly messager: Messager,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  execute = async (
    command: UpdateConfigurationCommand,
  ): Promise<Configuration> => {
    const entity = this.mapper.map(
      command.updateConfigurationRequest,
      UpdateConfigurationRequest,
      Configuration,
    );

    try {
      const configuration = await this.repository.update(
        command.updateConfigurationRequest.uuid,
        entity,
      );
      this.messager.publish(
        'configuration.update',
        JSON.stringify(
          this.mapper.map(
            configuration,
            Configuration,
            ConfigurationMessagerPresenter,
          ),
        ),
      );
      this.messager.publish(
        'logging.configuration.update.info',
        JSON.stringify(command.updateConfigurationRequest),
      );
      return configuration;
    } catch (error) {
      this.messager.publish(
        'logging.configuration.update.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
